package at.ac.uibk.gitsearch.service;

import static org.hamcrest.Matchers.anyOf;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.everyItem;
import static org.hamcrest.Matchers.hasProperty;

import java.io.IOException;
import java.io.InputStream;

import org.apache.logging.log4j.LogManager;
import org.codeability.sharing.plugins.api.SharingPluginConfig;
import org.codeability.sharing.plugins.api.SharingPluginConfig.Action;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.FileCopyUtils;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.service.PluginManagementService.PluginConfigWrapper;
import at.ac.uibk.gitsearch.service.PluginManagementService.PluginConfigWrapper.ActionWrapper;
import at.ac.uibk.gitsearch.service.dto.SearchInputDTO;
import at.ac.uibk.gitsearch.service.dto.SearchInputMetadataDTO;
import at.ac.uibk.gitsearch.service.dto.SearchResultDTO;
import at.ac.uibk.gitsearch.service.dto.SearchResultsDTO;
import at.ac.uibk.gitsearch.service.dto.UserProvidedMetadataDTO;

@SpringBootTest(classes = GitsearchApp.class)
public class SearchServiceIT {

	private static  final Logger LOGGER = LoggerFactory.getLogger(SearchServiceIT.class);
	
	@Autowired
	private SearchService searchService;


	private PluginConfigWrapper config;
	@BeforeEach
	public void initConfig() {
		config = new PluginConfigWrapper(
				new SharingPluginConfig("TestPlugin", new Action[] { new Action("someId", "someImportURL",
						"someActionName", "metadata.type.externalName=='programming exercise'") }),
				"http://localhost:8080/xxx");
	}

	@Test
	public void testExpressionFiltersOnMetaData() throws JsonParseException, JsonMappingException, IOException {

		ActionWrapper actionWrapper = config.getActions().get("someId");

		Assert.assertTrue("metaData1 should be applicable",
				actionWrapper.isApplicable(readTestResults("metaData1.yaml")));
		Assert.assertFalse("metaData2 should not be applicable",
				actionWrapper.isApplicable(readTestResults("metaData2.yaml")));

	}

	/**
	 * temporary static test data.
	 *
	 * @param infoString just a string to add to testdata description.
	 * @return
	 * @throws IOException
	 * @throws JsonMappingException
	 * @throws JsonParseException
	 */
	private SearchResultDTO readTestResults(String testFile)
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
		mapper.findAndRegisterModules();

		LOGGER.debug("reading test data from {} ", testFile);

		final String resourceName = "./testData/" + testFile;
		final InputStream resourceStream = this.getClass().getResourceAsStream(resourceName);
		Assert.assertNotNull("cannot find resource at " + resourceName, resourceStream);

		byte[] fileContent = FileCopyUtils.copyToByteArray(resourceStream);
		UserProvidedMetadataDTO metaData = mapper.readValue(fileContent, UserProvidedMetadataDTO.class);
		SearchResultDTO result = new SearchResultDTO();
		result.setMetadata(metaData);
		return result;

	}

	
	@Test
	public void testFullTextSearch() throws Exception {
		LOGGER.info("starting testFullTextSearch");
		final String WIEN = "Wien";
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, null, null);
		SearchInputDTO searchQuery = new SearchInputDTO(WIEN, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
		org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		org.junit.Assert.assertThat(searchResultPage.getSearchResult(),
				everyItem(anyOf(hasProperty("metadata", hasProperty("description", containsString(WIEN))),
						hasProperty("metadata", hasProperty("title", containsString(WIEN)))
				// hasProperty("keywords", containsString("Wien"))
				)));

		LOGGER.info("exiting testFullTextSearch");
	}


}
